#
#   SCOREMIPSUM
#
"""
init
----------

init for the `scoremipsum` module.
"""

__version__ = "0.0.6"


def game():
    return None


def generation():
    return None


def score():
    return None
